@@ -348,8 +348,8 @@ elf_read_page(struct page_io *pio)
 			if (size > loadaddr + pls->filesz - addr)
 				size = loadaddr + pls->filesz - addr;
 
-			status = fcache_pread(ctx->shared->fcache,
-					      p, size, 0, pos);
+			status = flatmap_pread(ctx->shared->flatmap, p, size,
+					       0, pos);
 			if (status != KDUMP_OK)
 				goto err_read;
 			p += size;
@@ -420,8 +420,8 @@ elf_get_page(struct page_io *pio)
 		return cache_get_page(pio, elf_read_page);
 
 	mutex_lock(&ctx->shared->cache_lock);
-	status = fcache_get_chunk(ctx->shared->fcache, &pio->chunk, sz,
-				  0, pls->file_offset + addr - loadaddr);
+	status = flatmap_get_chunk(ctx->shared->flatmap, &pio->chunk, sz,
+				   0, pls->file_offset + addr - loadaddr);
 	mutex_unlock(&ctx->shared->cache_lock);
 	return status;
 }
@@ -915,7 +915,7 @@ xc_p2m_first_step(addrxlat_step_t *step, addrxlat_addr_t addr)
 
 	pos = edp->xen_map_offset + idx * sizeof(struct xen_p2m);
 	mutex_lock(&shared->cache_lock);
-	status = fcache_pread(shared->fcache, &p2m, sizeof p2m, 0, pos);
+	status = flatmap_pread(shared->flatmap, &p2m, sizeof p2m, 0, pos);
 	mutex_unlock(&shared->cache_lock);
 	if (status != KDUMP_OK)
 		return addrxlat_read_error(step->ctx, "p2m entry", pos);
@@ -951,7 +951,7 @@ xc_m2p_first_step(addrxlat_step_t *step, addrxlat_addr_t addr)
 
 	pos = edp->xen_map_offset + idx * sizeof(struct xen_p2m);
 	mutex_lock(&shared->cache_lock);
-	status = fcache_pread(shared->fcache, &p2m, sizeof p2m, 0, pos);
+	status = flatmap_pread(shared->flatmap, &p2m, sizeof p2m, 0, pos);
 	mutex_unlock(&shared->cache_lock);
 	if (status != KDUMP_OK)
 		return addrxlat_read_error(step->ctx, "m2p entry", pos);
@@ -1054,8 +1054,8 @@ xc_get_page(struct page_io *pio)
 	offset = edp->xen_pages_offset + ((off_t)idx << get_page_shift(ctx));
 
 	mutex_lock(&ctx->shared->cache_lock);
-	status = fcache_get_chunk(ctx->shared->fcache, &pio->chunk,
-				  get_page_size(ctx), 0, offset);
+	status = flatmap_get_chunk(ctx->shared->flatmap, &pio->chunk,
+				   get_page_size(ctx), 0, offset);
 	mutex_unlock(&ctx->shared->cache_lock);
 	return status;
 }
@@ -1155,8 +1155,8 @@ init_strtab(kdump_ctx_t *ctx, unsigned strtabidx)
 	if (!edp->strtab)
 		return KDUMP_ERR_SYSTEM;
 
-	status = fcache_pread(ctx->shared->fcache, edp->strtab,
-			      ps->size, 0, ps->file_offset);
+	status = flatmap_pread(ctx->shared->flatmap, edp->strtab, ps->size,
+			       0, ps->file_offset);
 	if (status != KDUMP_OK) {
 		free(edp->strtab);
 		edp->strtab = NULL;
@@ -1193,9 +1193,9 @@ init_elf32(kdump_ctx_t *ctx, Elf32_Ehdr *ehdr)
 	if (offset != 0 && (shnum == 0 || phnum == PN_XNUM)) {
 		Elf32_Shdr *sect;
 
-		ret = fcache_get_chunk(ctx->shared->fcache, &fch,
-				       dump16toh(ctx, ehdr->e_shentsize),
-				       0, offset);
+		ret = flatmap_get_chunk(ctx->shared->flatmap, &fch,
+					dump16toh(ctx, ehdr->e_shentsize),
+					0, offset);
 		if (ret != KDUMP_OK)
 			return set_hdr_error(ctx, ret, "section", 0, offset);
 		sect = (Elf32_Shdr*) fch.data;
@@ -1222,8 +1222,8 @@ init_elf32(kdump_ctx_t *ctx, Elf32_Ehdr *ehdr)
 		Elf32_Phdr *prog;
 		struct load_segment *pls;
 
-		ret = fcache_get_chunk(ctx->shared->fcache, &fch,
-				       entsz, 0, offset);
+		ret = flatmap_get_chunk(ctx->shared->flatmap, &fch, entsz,
+					0, offset);
 		if (ret != KDUMP_OK)
 			return set_hdr_error(ctx, ret, "program", i, offset);
 		offset += entsz;
@@ -1247,8 +1247,8 @@ init_elf32(kdump_ctx_t *ctx, Elf32_Ehdr *ehdr)
 	for (i = 0; i < shnum; ++i) {
 		Elf32_Shdr *sect;
 
-		ret = fcache_get_chunk(ctx->shared->fcache, &fch,
-				       entsz, 0, offset);
+		ret = flatmap_get_chunk(ctx->shared->flatmap, &fch, entsz,
+					0, offset);
 		if (ret != KDUMP_OK)
 			return set_hdr_error(ctx, ret, "section", i, offset);
 		offset += entsz;
@@ -1286,9 +1286,9 @@ init_elf64(kdump_ctx_t *ctx, Elf64_Ehdr *ehdr)
 	if (offset != 0 && (shnum == 0 || phnum == PN_XNUM)) {
 		Elf64_Shdr *sect;
 
-		ret = fcache_get_chunk(ctx->shared->fcache, &fch,
-				       dump16toh(ctx, ehdr->e_shentsize),
-				       0, offset);
+		ret = flatmap_get_chunk(ctx->shared->flatmap, &fch,
+					dump16toh(ctx, ehdr->e_shentsize),
+					0, offset);
 		if (ret != KDUMP_OK)
 			return set_hdr_error(ctx, ret, "section", 0, offset);
 		sect = (Elf64_Shdr*) fch.data;
@@ -1315,8 +1315,8 @@ init_elf64(kdump_ctx_t *ctx, Elf64_Ehdr *ehdr)
 		Elf64_Phdr *prog;
 		struct load_segment *pls;
 
-		ret = fcache_get_chunk(ctx->shared->fcache, &fch,
-				       entsz, 0, offset);
+		ret = flatmap_get_chunk(ctx->shared->flatmap, &fch, entsz,
+					0, offset);
 		if (ret != KDUMP_OK)
 			return set_hdr_error(ctx, ret, "program", i, offset);
 		offset += entsz;
@@ -1340,8 +1340,8 @@ init_elf64(kdump_ctx_t *ctx, Elf64_Ehdr *ehdr)
 	for (i = 0; i < shnum; ++i) {
 		Elf64_Shdr *sect;
 
-		ret = fcache_get_chunk(ctx->shared->fcache, &fch,
-				       entsz, 0, offset);
+		ret = flatmap_get_chunk(ctx->shared->flatmap, &fch, entsz,
+					0, offset);
 		if (ret != KDUMP_OK)
 			return set_hdr_error(ctx, ret, "section", i, offset);
 		offset += entsz;
@@ -1374,8 +1374,8 @@ walk_elf_notes(kdump_ctx_t *ctx, walk_notes_fn *fn)
 	for (i = 0; i < edp->num_note_segments; ++i) {
 		struct load_segment *seg = edp->note_segments + i;
 
-		ret = fcache_get_chunk(ctx->shared->fcache, &fch,
-				       seg->filesz, 0, seg->file_offset);
+		ret = flatmap_get_chunk(ctx->shared->flatmap, &fch,
+					seg->filesz, 0, seg->file_offset);
 		if (ret != KDUMP_OK)
 			return set_read_error(ctx, ret, "ELF notes",
 					      seg->file_offset);
@@ -1532,8 +1532,9 @@ open_common(kdump_ctx_t *ctx)
 				return set_error(ctx, ret,
 						 "Cannot create Xen PFN map");
 		} else if (!strcmp(name, ".note.Xen")) {
-			ret = fcache_get_chunk(ctx->shared->fcache, &fch,
-					       sect->size, 0, sect->file_offset);
+			ret = flatmap_get_chunk(ctx->shared->flatmap, &fch,
+						sect->size, 0,
+						sect->file_offset);
 			if (ret != KDUMP_OK)
 				return set_error(ctx, ret,
 						 "Cannot read '%s'", name);
@@ -1543,8 +1544,9 @@ open_common(kdump_ctx_t *ctx)
 				return set_error(ctx, ret,
 						 "Cannot process Xen notes");
 		} else if (!strcmp(name, ".xen_prstatus")) {
-			ret = fcache_get_chunk(ctx->shared->fcache, &fch,
-					       sect->size, 0, sect->file_offset);
+			ret = flatmap_get_chunk(ctx->shared->flatmap, &fch,
+						sect->size, 0,
+						sect->file_offset);
 			if (ret != KDUMP_OK)
 				return set_error(ctx, ret,
 						 "Cannot read '%s'", name);
@@ -1589,21 +1591,15 @@ open_common(kdump_ctx_t *ctx)
 static kdump_status
 do_probe(kdump_ctx_t *ctx, void *hdr)
 {
+	struct elfdump_priv *edp = ctx->shared->fmtdata;
 	unsigned char *eheader = hdr;
 	Elf32_Ehdr *elf32 = hdr;
 	Elf64_Ehdr *elf64 = hdr;
-	struct elfdump_priv *edp;
 
 	if (memcmp(eheader, ELFMAG, SELFMAG))
 		return set_error(ctx, KDUMP_NOPROBE,
 				 "Invalid ELF signature");
 
-	edp = calloc(1, sizeof *edp);
-	if (!edp)
-		return set_error(ctx, KDUMP_ERR_SYSTEM,
-				 "Cannot allocate ELF dump private data");
-	ctx->shared->fmtdata = edp;
-
 	switch (eheader[EI_DATA]) {
 	case ELFDATA2LSB:
 		set_byte_order(ctx, KDUMP_LITTLE_ENDIAN);
@@ -1638,11 +1634,18 @@ do_probe(kdump_ctx_t *ctx, void *hdr)
 static kdump_status
 elf_probe(kdump_ctx_t *ctx)
 {
+	struct elfdump_priv *edp;
 	struct fcache_chunk fch;
 	kdump_status ret;
 
-	ret = fcache_get_chunk(ctx->shared->fcache, &fch,
-			       sizeof(Elf64_Ehdr), 0, 0);
+	edp = calloc(1, sizeof *edp);
+	if (!edp)
+		return set_error(ctx, KDUMP_ERR_SYSTEM,
+				 "Cannot allocate ELF dump private data");
+	ctx->shared->fmtdata = edp;
+
+	ret = flatmap_get_chunk(ctx->shared->flatmap, &fch, sizeof(Elf64_Ehdr),
+				0, 0);
 	if (ret != KDUMP_OK)
 		return set_error(ctx, ret, "Cannot read dump header");
 
