@@ -221,7 +221,12 @@ struct arch_ops {
 	kdump_status (*init)(kdump_ctx_t *);
 
 	/** Process an NT_PRSTATUS note. */
-	kdump_status (*process_prstatus)(kdump_ctx_t *, const void *, size_t);
+	kdump_status (*process_prstatus)(kdump_ctx_t *ctx, unsigned int cpu,
+					 const void *data, size_t size);
+
+	/** Process a QEMU CPU state note. */
+	kdump_status (*process_qemu_cpustate)(kdump_ctx_t *ctx, unsigned int cpu,
+					      const void *data, size_t size);
 
 	/** Process a Xen .xen_prstatus section. */
 	kdump_status (*process_xen_prstatus)(kdump_ctx_t *, const void *, size_t);
@@ -450,11 +455,11 @@ struct attr_template {
 /**  Attribute value flags.
  */
 struct attr_flags {
-	uint8_t isset : 1;	/**< Zero if attribute has no value */
-	uint8_t persist : 1;	/**< Persistent (not cleared on re-open) */
-	uint8_t dynstr : 1;	/**< Dynamically allocated string */
-	uint8_t indirect : 1;	/**< Actual value is at @c *pval */
-	uint8_t invalid : 1;	/**< Value needs revalidation */
+	bool isset : 1;		/**< Zero if attribute has no value */
+	bool persist : 1;	/**< Persistent (not cleared on re-open) */
+	bool dynstr : 1;	/**< Dynamically allocated string */
+	bool indirect : 1;	/**< Actual value is at @c *pval */
+	bool invalid : 1;	/**< Value needs revalidation */
 };
 
 /**  Default attribute flags. */
@@ -463,20 +468,23 @@ struct attr_flags {
 
 /**  Persistent attribute flags. */
 #define ATTR_PERSIST	\
-	((struct attr_flags){ .persist = 1 })
+	((struct attr_flags){ .persist = true })
 
 /**  Indirect attribute flags. */
 #define ATTR_INDIRECT	\
-	((struct attr_flags){ .indirect = 1 })
+	((struct attr_flags){ .indirect = true })
 
 /**  Invalid attribute flags. */
 #define ATTR_INVALID	\
-	((struct attr_flags){ .invalid = 1 })
+	((struct attr_flags){ .invalid = true })
 
 /**  Persistent indirect attribute flags. */
 #define ATTR_PERSIST_INDIRECT	\
-	((struct attr_flags){ .persist = 1, .indirect = 1 })
+	((struct attr_flags){ .persist = true, .indirect = true })
 
+/** Dynamically allocated attribute flags. */
+#define ATTR_DYNSTR	\
+	((struct attr_flags){ .dynstr = true })
 
 /**  Attribute template flags.
  */
@@ -580,6 +588,9 @@ struct kdump_shared {
 	struct fcache *fcache;	/**< File cache. */
 	mutex_t cache_lock;	/**< Cache access lock. */
 
+	/** File offset mappings for flattened files. */
+	struct flattened_map *flatmap;
+
 	/** Static attributes. */
 #define ATTR(dir, key, field, type, ctype, ...)	\
 	kdump_attr_value_t field;
@@ -859,6 +870,12 @@ INTERNAL_DECL(kdump_status, create_cpu_regs,
 	      (kdump_ctx_t *ctx, unsigned cpu,
 	       struct derived_attr_def *def, unsigned ndef));
 
+INTERNAL_DECL(kdump_status, init_qemu_cpustate,
+	      (kdump_ctx_t *ctx, unsigned cpu, const void *data, size_t size));
+INTERNAL_DECL(kdump_status, create_qemu_cpu_regs,
+	      (kdump_ctx_t *ctx, unsigned cpu,
+	       struct derived_attr_def *def, unsigned ndef));
+
 INTERNAL_DECL(kdump_status, init_xen_cpu_prstatus,
 	      (kdump_ctx_t *ctx, unsigned cpu, const void *data, size_t size));
 INTERNAL_DECL(kdump_status, create_xen_cpu_regs,
@@ -1062,8 +1079,6 @@ attr_dict_decref(struct attr_dict *dict)
 	return 0;
 }
 
-DECLARE_ALIAS(get_attr);
-
 /**  Attribute data by dict and global key index.
  * @param dict  Attribute dictionary.
  * @param idx   Global key index.
@@ -1088,9 +1103,9 @@ gattr(const kdump_ctx_t *ctx, enum global_keyidx idx)
 
 /**  Check if an attribute is set.
  * @param data  Pointer to the attribute data.
- * @returns     Non-zero if attribute data is valid.
+ * @returns     @c true if attribute data is valid.
  */
-static inline int
+static inline bool
 attr_isset(const struct attr_data *data)
 {
 	return data->flags.isset;
@@ -1119,7 +1134,7 @@ attr_embed_value(struct attr_data *attr)
 {
 	if (attr->flags.indirect) {
 		attr->val = *attr->pval;
-		attr->flags.indirect = 0;
+		attr->flags.indirect = false;
 	}
 }
 
@@ -1737,6 +1752,94 @@ find_pfn_file_map(const struct pfn_file_map *maps, size_t nmaps,
 	return NULL;
 }
 
+/* Flattened files. */
+
+/** Offset mapping for a file in the flattened format. */
+struct flattened_file_map {
+	/** Map (rearranged) offset to an index in the offset arrray. */
+	addrxlat_map_t *map;
+
+	/** Differences between flattened and rearranged file offsets. */
+	off_t *offs;
+};
+
+/** Offset mappings for a set of flattened files. */
+struct flattened_map {
+	/** File cache. */
+	struct fcache *fcache;
+
+	/** Number of mapped files. */
+	unsigned nfiles;
+
+	/** Mappings for individual files. */
+	struct flattened_file_map fmap[];
+};
+
+INTERNAL_DECL(struct flattened_map *, flatmap_alloc,
+	      (unsigned nfiles));
+INTERNAL_DECL(kdump_status, flatmap_init,
+	      (struct flattened_map *map, kdump_ctx_t *ctx));
+INTERNAL_DECL(void, flatmap_free,
+	      (struct flattened_map *map));
+INTERNAL_DECL(kdump_status, flatmap_pread_flat,
+	      (struct flattened_map *map, void *buf, size_t len,
+	       unsigned fidx, off_t pos));
+INTERNAL_DECL(kdump_status, flatmap_get_chunk_flat,
+	      (struct flattened_map *map, struct fcache_chunk *fch,
+	       size_t len, unsigned fidx, off_t pos));
+
+/** Check whether a given file in a set is flattened.
+ * @param map   Flattened offset map.
+ * @param fidx  Index of the file to read from.
+ * @returns     @c true if file is flattened, @c false otherwise.
+ */
+static inline bool
+flatmap_isflattened(struct flattened_map *map, unsigned fidx)
+{
+	return fidx < map->nfiles && map->fmap[fidx].map;
+}
+
+/** Read buffer from a possibly flattened dump file.
+ * @param flatmap  Flattened offset map.
+ * @param buf      Target I/O buffer.
+ * @param len      Length of data.
+ * @param fidx     Index of the file to read from.
+ * @param pos      File position.
+ * @returns        Error status.
+ *
+ * Read data from an ELF dump file. If the file is flattened, interpret
+ * @p pos as if it was already rearranged.
+ */
+static inline kdump_status
+flatmap_pread(struct flattened_map *flatmap, void *buf, size_t len,
+	      unsigned fidx, off_t pos)
+{
+	return flatmap_isflattened(flatmap, fidx)
+		? flatmap_pread_flat(flatmap, buf, len, fidx, pos)
+		: fcache_pread(flatmap->fcache, buf, len, fidx, pos);
+}
+
+/** Get a contiguous data chunk from a possibly flattend dump file.
+ * @param flatmap  Flattened offset map.
+ * @param fch      File cache chunk, updated on success.
+ * @param len      Length of data.
+ * @param fidx     Index of the file to read from.
+ * @param pos      File position.
+ * @returns        Error status.
+ *
+ * Get a contiguous data chunk from an ELF dump file. If the file is
+ * flattened, interpret @p pos as if it was already rearranged.
+ */
+static inline kdump_status
+flatmap_get_chunk(struct flattened_map *flatmap, struct fcache_chunk *fch,
+		  size_t len, unsigned fidx, off_t pos)
+{
+	return flatmap_isflattened(flatmap, fidx)
+		? flatmap_get_chunk_flat(flatmap, fch, len, fidx, pos)
+		: fcache_get_chunk(flatmap->fcache, fch, len, fidx, pos);
+}
+
+
 /** Check if a character is a POSIX white space.
  * @param c  Character to check.
  *
