@@ -173,9 +173,20 @@ process_core_note(kdump_ctx_t *ctx, uint32_t type,
 		  void *desc, size_t descsz)
 {
 	if (type == NT_PRSTATUS) {
+		unsigned int cpu;
+		kdump_status status;
+
+		cpu = isset_num_cpus(ctx) ? get_num_cpus(ctx) : 0;
+		set_num_cpus(ctx, cpu + 1);
+
+		status = init_cpu_prstatus(ctx, cpu, desc, descsz);
+		if (status != KDUMP_OK)
+			return set_error(ctx, status, "Cannot set CPU %u %s",
+					 cpu, "PRSTATUS");
+
 		if (ctx->shared->arch_ops && ctx->shared->arch_ops->process_prstatus)
 			return ctx->shared->arch_ops->process_prstatus(
-				ctx, desc, descsz);
+				ctx, cpu, desc, descsz);
 	} else if (type == NT_TASKSTRUCT) {
 		return process_task_struct(ctx, desc, descsz);
 	}
@@ -315,6 +326,36 @@ process_xen_dumpcore_version(kdump_ctx_t *ctx, void *data, size_t len)
 	return KDUMP_OK;
 }
 
+/** QEMU ELF note types. */
+enum {
+	QEMU_ELFNOTE_CPUSTATE = 0,
+};
+
+static kdump_status
+process_qemu_note(kdump_ctx_t *ctx, uint32_t type,
+		  void *desc, size_t descsz)
+{
+	if (type == QEMU_ELFNOTE_CPUSTATE) {
+		unsigned int cpu;
+		kdump_status status;
+
+		cpu = isset_num_qemu_cpus(ctx) ? get_num_qemu_cpus(ctx) : 0;
+		set_num_qemu_cpus(ctx, cpu + 1);
+
+		status = init_qemu_cpustate(ctx, cpu, desc, descsz);
+		if (status != KDUMP_OK)
+			return set_error(ctx, status, "Cannot set CPU %u %s",
+					 cpu, "QEMU_CPUSTATE");
+
+		if (ctx->shared->arch_ops &&
+		    ctx->shared->arch_ops->process_qemu_cpustate)
+			return ctx->shared->arch_ops->process_qemu_cpustate(
+				ctx, cpu, desc, descsz);
+	}
+
+	return KDUMP_OK;
+}
+
 /* These fields in kdump_ctx_t must be initialised:
  *
  *   endian
@@ -398,6 +439,8 @@ do_arch_note(kdump_ctx_t *ctx, Elf32_Word type,
 {
 	if (note_equal("CORE", name, namesz))
 		return process_core_note(ctx, type, desc, descsz);
+	else if (note_equal("QEMU", name, namesz))
+		return process_qemu_note(ctx, type, desc, descsz);
 	else if (note_equal("Xen", name, namesz))
 		return process_xen_note(ctx, type, desc, descsz);
 	else if (note_equal(".note.Xen", name, namesz))
