@@ -193,7 +193,7 @@ num_files_pre_hook(kdump_ctx_t *ctx, struct attr_data *attr,
 			break;
 		}
 		if (i == 0) {
-			fdattr->flags.indirect = 1;
+			fdattr->flags.indirect = true;
 			fdattr->pval = attr_mut_value(gattr(ctx, GKI_file_fd));
 		}
 	}
@@ -290,6 +290,7 @@ open_dump(kdump_ctx_t *ctx)
 	int fdset[nfiles];
 	int i;
 
+	flatmap_free(ctx->shared->flatmap);
 	if (ctx->shared->fcache) {
 		for (i = 0; i < ARRAY_SIZE(fcache_attrs); ++i)
 			attr_embed_value(gattr(ctx, fcache_attrs[i]));
@@ -321,6 +322,15 @@ open_dump(kdump_ctx_t *ctx)
 			gattr(ctx, GKI_read_cache_hits),
 			gattr(ctx, GKI_read_cache_misses));
 
+	ctx->shared->flatmap = flatmap_alloc(nfiles);
+	if (!ctx->shared->flatmap)
+		return set_error(ctx, KDUMP_ERR_SYSTEM,
+				 "Cannot allocate %s", "flattened dump maps");
+
+	ret = flatmap_init(ctx->shared->flatmap, ctx);
+	if (ret != KDUMP_OK)
+		return ret;
+
 	ctx->xlat->dirty = true;
 
 	for (i = 0; i < ARRAY_SIZE(formats); ++i) {
@@ -328,6 +338,8 @@ open_dump(kdump_ctx_t *ctx)
 		ret = ctx->shared->ops->probe(ctx);
 		if (ret == KDUMP_OK)
 			return finish_open_dump(ctx);
+		if (ctx->shared->ops->cleanup)
+			ctx->shared->ops->cleanup(ctx->shared);
 		if (ret != KDUMP_NOPROBE)
 			return ret;
 
